import sqlite3
import pkg_resources

DB_FILE = "data/zeitzono.sqlite"

MAX_LIMIT_AMT = 100


class ZeitzonoDB:
    def __init__(self):

        dbfile = pkg_resources.resource_filename("Zeitzono", DB_FILE)
        self.conn = sqlite3.connect(dbfile)

    def _sql_gen(
        self,
        term1,
        term2=None,
        subterm=None,
        count=False,
        limit=False,
        limitamt=25,
    ):

        select_line = """select cities.asciiname, countries.name,
                                admin1.name, admin2.name, cities.timezone"""

        if count:
            select_line = "select count(asciiname)"

        from_line = "from cities"

        on_line = """left join countries on (cities.countrycode = countries.countrycode)
                     left join admin1 on (admin1.code = (cities.countrycode || '.' || cities.admin1))
                     left join admin2 on (admin2.code = (cities.countrycode || '.' || cities.admin1 || '.' || cities.admin2))
                  """  # noqa

        if term2 is None and subterm is None:
            where_line = """
                         where cities.alternatenames like ?
                         OR
                         cities.name like ?
                         OR
                         cities.asciiname like ?
            """

        if term2 is not None and subterm is None:
            where_line = """where
                            (cities.alternatenames like ?
                            OR
                            cities.name like ?
                            OR
                            cities.asciiname like ?)
                            AND
                            (cities.alternatenames like ?
                            OR
                            cities.name like ?
                            OR
                            cities.asciiname like ?)
                        """

        if term2 is None and subterm is not None:
            where_line = """
                         where (cities.alternatenames like ?
                         OR
                         cities.name like ?
                         OR
                         cities.asciiname like ?)
                         AND
                         (countries.name like ?
                         OR
                         admin1.name like ?
                         OR
                         admin2.name like ?)
                         """

        if term2 is not None and subterm is not None:
            where_line = """where
                            (cities.alternatenames like ?
                            OR
                            cities.name like ?
                            OR
                            cities.asciiname like ?)
                            AND
                            (cities.alternatenames like ?
                            OR
                            cities.name like ?
                            OR
                            cities.asciiname like ?)
                            AND
                            (countries.name like ?
                            OR
                            admin1.name like ?
                            OR
                            admin2.name like ?)
                         """

        order_line = "order by population desc"
        if count:
            order_line = ""

        limit_line = ""
        if limit:
            if limitamt > MAX_LIMIT_AMT:
                limitamt = MAX_LIMIT_AMT
            limit_line = "limit %s" % limitamt

        sql = "\n".join(
            [
                select_line,
                from_line,
                on_line,
                where_line,
                order_line,
                limit_line,
            ]
        )

        return sql

    def db_count(self, term1, term2=None, subterm=None):
        return self.db_search(term1, term2, subterm, count=True)

    def db_search(
        self,
        term1,
        term2=None,
        subterm=None,
        count=False,
        limit=False,
        limitamt=25,
    ):
        sql = self._sql_gen(term1, term2, subterm, count, limit)
        # print(sql, file=open("foo", "a"))
        cursor = self.conn.cursor()

        if term2 is None and subterm is None:
            results = cursor.execute(
                sql, ("%" + term1 + "%", "%" + term1 + "%", "%" + term1 + "%")
            )
        elif term2 is not None and subterm is None:
            results = cursor.execute(
                sql,
                (
                    "%" + term1 + "%",
                    "%" + term1 + "%",
                    "%" + term1 + "%",
                    "%" + term2 + "%",
                    "%" + term2 + "%",
                    "%" + term2 + "%",
                ),
            )
        elif term2 is None and subterm is not None:
            results = cursor.execute(
                sql,
                (
                    "%" + term1 + "%",
                    "%" + term1 + "%",
                    "%" + term1 + "%",
                    "%" + subterm + "%",
                    "%" + subterm + "%",
                    "%" + subterm + "%",
                ),
            )
        elif term2 is not None and subterm is not None:
            results = cursor.execute(
                sql,
                (
                    "%" + term1 + "%",
                    "%" + term1 + "%",
                    "%" + term1 + "%",
                    "%" + term2 + "%",
                    "%" + term2 + "%",
                    "%" + term2 + "%",
                    "%" + subterm + "%",
                    "%" + subterm + "%",
                    "%" + subterm + "%",
                ),
            )

        if count:
            return cursor.fetchone()[0]

        return reversed(results.fetchall())

    def random_cities(self, count):
        rsql = """
                      select cities.asciiname, countries.name, admin1.name, admin2.name, cities.timezone
                      from cities
                      left join countries on (cities.countrycode = countries.countrycode)
                      left join admin1 on (admin1.code = (cities.countrycode || '.' || cities.admin1))
                      left join admin2 on (admin2.code = (cities.countrycode || '.' || cities.admin1 || '.' || cities.admin2))
                      where cities.rowid in (select rowid from cities order by random() limit %s)"""  # noqa
        rsql = rsql % count
        cursor = self.conn.cursor()
        results = cursor.execute(rsql)
        return results.fetchall()



from .ZeitzonoDB import ZeitzonoDB
from .ZeitzonoCities import ZeitzonoCities
from .ZeitzonoCity import ZeitzonoCity


class ZeitzonoSearch:
    """
    will search for city names
    returns Cities object with number of results found

    """

    def __init__(self):
        self.hdb = ZeitzonoDB()

    def _format_db_results(self, results, numresults):

        cities = []
        for result in results:
            name = result[0]
            country = result[1]
            admin1 = result[2] or ""  # needed to convert None to empty string
            admin2 = result[3] or ""  # needed to convert None to empty string
            tz = result[4]
            city = ZeitzonoCity(name, country, admin1, admin2, tz)
            cities.append(city)

        return ZeitzonoCities(cities=cities, nresults=numresults)

    def search(self, term1=None, term2=None, subterm=None, limitamt=25):
        term1_l = term1.lower() if term1 is not None else None
        term2_l = term2.lower() if term2 is not None else None
        subterm_l = subterm.lower() if subterm is not None else None

        if (term1 is None) or (len(term1.strip()) == 0):
            return ZeitzonoCities(nresults=0)

        numresults = self.hdb.db_count(term1_l, term2_l, subterm_l)

        if numresults <= 0:
            return ZeitzonoCities(nresults=0)

        limit = False
        if numresults > limitamt:
            limit = True

        results = self.hdb.db_search(
            term1_l, term2_l, subterm_l, limit=limit, limitamt=limitamt
        )

        return self._format_db_results(results, numresults)

    def random1(self):
        results = self.hdb.random_cities(1)
        return self._format_db_results(results, None)

    def random10(self):
        results = self.hdb.random_cities(10)
        return self._format_db_results(results, None)
